/***************************************************************************
  MLX90393.ino
  Code by lingib https://www.instructables.com/
  Last update 2 April 2022

  ------
  ABOUT
  ------

  This code computes the absolute angle of rotation of a magnet centered
  over a MLX90393 3-axis magnetometer.

  Rotate the magnet one whole revolution to calibrate.

  The setup() values are from the Adafruit_MLX90393\examples\basicdemo.ino library file.

  ----------
  COPYRIGHT
  ----------
  This code is free software: you can redistribute it and/or
  modify it under the terms of the GNU General Public License as published
  by the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This software is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License. If
  not, see <http://www.gnu.org/licenses/>.
***************************************************************************/

#include "Adafruit_MLX90393.h"

Adafruit_MLX90393 sensor = Adafruit_MLX90393();
#define MLX90393_CS 10

// ----- Auto-calibration
float
Xmax,
Xmin,
Xmid,
Xpeak;

float
Ymax,
Ymin,
Ymid,
Ypeak,
Ynormal;

float
Angle;

// ----- Software timer
unsigned long Timer = 0L;
unsigned long Delay = 2000L;

bool Debug = false;

// --------------
//  setup()
// --------------
void setup(void)
{
  Serial.begin(115200);

  // ----- Wait for serial on USB platforms. */
  while (!Serial) {
    delay(100);
  }

  // ----- Initialse search values
  Xmax = -10000.0;
  Xmin = 10000.0;
  Ymax = -10000.0;
  Ymin = 10000.0;

  // ----- Detect sensor
  if (! sensor.begin_I2C()) {
    Serial.println("No sensor found ... check your wiring?");
    while (1) {
      delay(10);
    }
  }

  // ----- Set Gain
  sensor.setGain(MLX90393_GAIN_2_5X);

  // ----- Set resolution, per axis
  sensor.setResolution(MLX90393_X, MLX90393_RES_19);
  sensor.setResolution(MLX90393_Y, MLX90393_RES_19);
  sensor.setResolution(MLX90393_Z, MLX90393_RES_16);

  // ----- Set oversampling
  sensor.setOversampling(MLX90393_OSR_2);

  // ----- Set digital filtering
  sensor.setFilter(MLX90393_FILTER_6);
}

// --------------
//  loop()
// --------------
void loop(void) {
  float x, y, z;

  // ---- Read X, Y, and Z, data
  if (sensor.readData(&x, &y, &z)) {

    // ----- Obtain Xpeak & Xmid values
    Xmax = max(Xmax, x);
    Xmin = min(Xmin, x);
    Xmid = (Xmax + Xmin) / 2;
    Xpeak = Xmax - Xmid;

    // ----- Obtain Ypeak & Ymid values
    Ymax = max(Ymax, y);
    Ymin = min(Ymin, y);
    Ymid = (Ymax + Ymin) / 2;
    Ypeak = Ymax - Ymid;

    // ----- Make X-axis & Y-axis peak amplitudes the same
    Ynormal = Xpeak / Ypeak;
  }

  // ----- Calculate the true X & Y values
  x = x - Xmid;
  y = (y - Ymid) * Ynormal;

  // ----- Calculate the absolute angle
  Angle = atan2(y, x) * RAD_TO_DEG;
  if (Angle < 0) Angle += 360;            // Needed as atan2 outputs negative values between 180~360 degrees

  // ----- Scale Angle for plotting
  Angle *= 10;                            // Amplitude range now 0~3600 to match MLX90393 microTesla  readings

  // ----- Display results
  if (Debug) {
    if (millis() > Timer) {
      Timer = millis() + Delay;

      // ----- Display calibration values results
      Serial.print(" Xmax: "); Serial.print(Xmax); Serial.print("\t"); Serial.print(" Ymax: "); Serial.println(Ymax);
      Serial.print(" Xmin: "); Serial.print(Xmin); Serial.print("\t"); Serial.print(" Ymin: "); Serial.println(Ymin);
      Serial.print(" Xmid: "); Serial.print(Xmid); Serial.print("\t"); Serial.print(" Ymid: "); Serial.println(Ymid);
      Serial.print("Xpeak: "); Serial.print(Xpeak); Serial.print("\t"); Serial.print("Ypeak: "); Serial.println(Ypeak);
      Serial.println(" ");
    }
  } else {

    // ----- Display/plot XY values and absolute rotation angle (scaled)
    Serial.print(x); Serial.print("\t"); Serial.print(y); Serial.print("\t"); Serial.println(Angle);
  }
}
